class Bolt {
	constructor(game, x, y, r, direction){
		this.game = game;
		
		// Rendering
		this.texture = game.textures.projectiles;
		this.map = game.atlas.projectiles.bolt;
		
		// Scale
		this.scale = 1;
		this.originalWidth = this.scale * 5 * 0.5;
		this.originalHeight = this.scale * 11;
		
		this.width = this.originalWidth;
		this.height = 0;
		this.maxHeight = this.originalHeight * 5;
		
		// Physics
		this.x = x;
		this.y = y;
		this.r = r;
		this.direction = direction;
		this.radius = 0;
		
		this.collision = {
			with: [ Alien ],
			width: 0,
			height: 0,
			radius: 0,
			offsetX: 0,
			offsetY: 0,
			hitRadius: 2,
			
			piercing: false,
			
			reaction: function(x, y, object, px, py){
				strongBurnReaction.call(this, x, y, object, px, py);
				
				if(!this.isAlive){
					for(var j=0; j<10; j++) game.scene.addParticle(new EnergyParticle(game, game.textures.projectiles, game.atlas.particles.boltTrail, x, y));
				}
			}
		};
		
		// Bolt
		this.maxLife = 0.5;
		this.life = this.maxLife;
		this.isAlive = true;
		
		this.boltTimer = 0;
	}
	
	
	update(delta){
		var speed = 84,
			angle = 45,
			rate = 0.8;
		
		if(this.isAlive && this.boltTimer >= rate){
			var offsetX = this.height/2 * Math.sin(this.r/180*Math.PI),
				offsetY = this.height/2 * -Math.cos(this.r/180*Math.PI);
			
			// Create the main fork
			var offsetR = Math.random() * (2 * angle) - angle,
				bolt = new Bolt(this.game, this.x + offsetX, this.y + offsetY, this.r + offsetR, this.direction);
			
			bolt.life = this.life;
			this.game.scene.addObject(bolt);
			
			// Create an additional fork
			if(Math.random() < this.life/this.maxLife){
				var offsetR = Math.random() * (2 * angle) - angle,
					bolt = new Bolt(this.game, this.x + offsetX, this.y + offsetY, this.r + offsetR, this.direction);
				
				bolt.life = this.life;
				this.game.scene.addObject(bolt);
			}
			
			this.isAlive = false;
		}
		
		
		var that = this;
		
		this.collision.offsetY = this.height/2;
		
		for(var i=0; i<3; i++){
			doMovement();
			doCollision(this);
		}
		
		// Create a particle
		if(this.game.canShowParticle(8)){
			this.game.scene.addParticle(new EnergyParticle(this.game, this.game.textures.projectiles, this.game.atlas.particles.boltTrail, this.x, this.y));
		}
		
		function doMovement(){
			// Grow the height
			if(that.isAlive){
				var offsetX = delta * speed * Math.sin(that.r/180*Math.PI),
				 	offsetY = delta * speed * -Math.cos(that.r/180*Math.PI);
				
				if(that.height!==that.maxHeight){
					that.height += delta * speed;
					that.height = Math.min(that.height, that.maxHeight);
					that.x += offsetX/2;
					that.y += offsetY/2;
				}
				
				else{
					that.x += offsetX;
					that.y += offsetY;
				}
				
				that.boltTimer += delta;
			}
			
			// Shrink the height
			else{
				that.x += delta * speed/2 * Math.sin(that.r/180*Math.PI);
				that.y += delta * speed/2 * -Math.cos(that.r/180*Math.PI);
				
				that.height -= delta * speed;
				if(that.height<=0) return that.life = 0;
			}
		}
		
		
		if(this.life - delta<0){
			this.life += delta;
			this.isAlive = false;
		}
		
		return this.life -= delta;
	}
	
	
	draw(context, textures, atlas){
		var texture = this.texture,
			map = this.map;
		
		context.save();
		context.translate(this.x, this.y);
		context.rotate(this.r/180*Math.PI);
		
		context.drawImage(
			texture,
			
			// Sprite Crop Location
			map[0],
			map[1] + (this.isAlive ? 0 : map[3] * (1 - this.height/this.maxHeight)),
			map[2],
			map[3] * this.height/this.maxHeight,
			
			// Screen Space Location
			-this.width/2,
			-this.height/2,
			this.width,
			this.height
		);

		context.restore();
	}
}
